library selectable_circle_list;

import 'dart:math';

import 'package:flutter/material.dart';
import 'package:selectable_circle/selectable_circle.dart';
import 'package:selectable_circle_list/selectable_circle_item.dart';

export 'package:selectable_circle_list/selectable_circle_item.dart';
import 'package:after_layout/after_layout.dart';

/// Widget that displays selectable Circles in a row
///
/// It can also have two rows when the selected item has subitems
class SelectableCircleList extends StatefulWidget {
  /// creates the Widget
  SelectableCircleList({
    @required this.children,
    this.description,
    this.subDescription,
    this.onTap,
    this.itemWidth,
  });

  /// a descrition that is displayed one row above the selectable items
  final Widget description;

  /// a descrition that is displayed one row above the selectable subitems
  final Widget subDescription;

  /// from these items the selectableCirles are built
  final List<SelectableCircleItem> children;

  /// when one of the circles is tapped this function is called
  final Function(String value, String subvalue) onTap;

  /// width of a item, if null its calculated that 4 circles fit the screen
  final double itemWidth;
  @override
  _SelectableCircleListState createState() => _SelectableCircleListState();
}

class _SelectableCircleListState extends State<SelectableCircleList>
    with AfterLayoutMixin<SelectableCircleList> {
  String _value = "first";
  double _descriptionHeight = 16.0;
  double _subDescriptionHeight = 16.0;
  final _descriptionKey = GlobalKey();
  final _subDescriptionKey = GlobalKey();
  @override
  Widget build(BuildContext context) {
    final descriptionContainer = Container(
      key: _descriptionKey,
      child: Align(alignment: Alignment.centerLeft, child: widget.description),
    );
    final subDescriptionContainer = Container(
        key: _subDescriptionKey,
        child: Align(
            alignment: Alignment.centerLeft, child: widget.subDescription));
    final selectedList =
        widget.children.where((item) => _value.startsWith("${item.value}"));
    final selected = selectedList.isNotEmpty ? selectedList.first : null;
    final needChilds = _value.isNotEmpty &&
        (selected != null) &&
        (selected.subItemList != null && selected.subItemList.isNotEmpty);
    final circleWidth =
        widget.itemWidth ?? (MediaQuery.of(context).size.width - 20) / 4;
    final rowHeight = circleWidth + 16.0;
    final completeHeight = rowHeight + _getHeightOfKey(_descriptionKey);
    final height = needChilds ? completeHeight * 2 : completeHeight;
    final out = Container(
      height: height,
      child: Column(
        children: <Widget>[
          descriptionContainer,
          Container(
            height: rowHeight,
            child: widget.children.length < 5
                ? Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: <Widget>[
                      ...widget.children
                          .map(
                            (sci) => _buildCircle(circleWidth, sci),
                          )
                          .toList(),
                    ],
                  )
                : ListView(
                    scrollDirection: Axis.horizontal,
                    children: [
                      ...widget.children
                          .map(
                            (sci) => _buildCircle(circleWidth, sci),
                          )
                          .toList(),
                    ],
                  ),
          ),
          if (needChilds) _buildSubitems(selected, subDescriptionContainer)
        ],
      ),
    );
    // Scrollable.ensureVisible(selected.key.currentContext);
    return out;
  }

  SelectableCircleList _buildSubitems(
      SelectableCircleItem selected, Widget subDescriptionContainer) {
    return SelectableCircleList(
      children: selected.subItemList
          .asMap()
          .map(
            (index, item) {
              final color = Color.lerp(
                selected.color,
                Colors.white,
                min(1.0, 0.9 * index / selected.subItemList.length),
              );
              return MapEntry(
                index,
                SelectableCircleItem(
                  selected.centerWidget,
                  item.description,
                  item.value,
                  color,
                ),
              );
            },
          )
          .values
          .toList(),
      description: subDescriptionContainer,
      onTap: _onTapChild,
    );
  }

  Padding _buildCircle(double circleWidth, SelectableCircleItem sci) {
    return Padding(
      padding: EdgeInsets.symmetric(horizontal: 5.0),
      child: SelectableCircle(
        width: circleWidth - 10,
        color: sci.color,
        borderColor: sci.color,
        selectMode: SelectMode.check,
        isSelected: _value.startsWith(sci.value),
        child: sci.centerWidget,
        bottomDescription: Text(sci.description),
        onTap: () {
          setState(() {
            _value = "${sci.value}";
          });
          widget.onTap(sci.value, "");
        },
      ),
    );
  }

  _onTapChild(String value, String subValue) {
    widget.onTap(_value, value);
  }

  double _getHeightOfKey(GlobalKey key) {
    final RenderBox renderBoxRed = key.currentContext?.findRenderObject();
    final size = renderBoxRed?.size;
    final height = size?.height;
    print("height $height");
    return height ?? 16.0;
  }

  @override
  void afterFirstLayout(BuildContext context) {
    final dh = _getHeightOfKey(_descriptionKey);
    final sdh = _getHeightOfKey(_subDescriptionKey);
    if ((sdh != _subDescriptionHeight) || (dh != _descriptionHeight)) {
      setState(() {
        _subDescriptionHeight = sdh;
        _descriptionHeight = dh;
      });
    }
  }
}
